/**
 * 
 */
package org.springblade.datasource.dialect;

import cn.hutool.core.util.StrUtil;
import com.mongodb.ConnectionString;
import com.mongodb.MongoClientSettings;
import com.mongodb.MongoCredential;
import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import org.bson.BsonDocument;
import org.bson.BsonValue;
import org.bson.Document;
import org.springblade.core.tool.utils.Func;
import org.springblade.datasource.AbstractGmetryDialect;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.etl.dto.FileStructure;
import org.springblade.etl.entity.bi.ReportDatasource;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author wookvn
 *
 */
@Component
public class MongoGmetryDialect extends AbstractGmetryDialect {

	@Override
	public List<String> buildCreateTableSql(String tableName, String tableComment, List<FileStructure> structureList) {
		return null;
	}

	@Override
	public List<TableColumnDto> listTableColumnsByTableViewName(ReportDatasource datasource, String tableViewName) {
		if(datasource == null || StrUtil.isBlank(tableViewName)){
			return null;
		}
		List<TableColumnDto> columnList = new ArrayList<>();
		try (MongoClient mongoClient = _createConnection(datasource)) {
			MongoDatabase mongoClientDatabase = mongoClient.getDatabase(datasource.getDsDb());
			MongoCollection<BsonDocument> collection = mongoClientDatabase.getCollection(tableViewName, BsonDocument.class);

			Document query = new Document();
			for(BsonDocument doc : collection.find(query).limit(1)) {
				for (Map.Entry<String, BsonValue> valueEntry : doc.entrySet()) {
					TableColumnDto tableColumnDto = new TableColumnDto();
					tableColumnDto.setColumnName(valueEntry.getKey());
					tableColumnDto.setDataType(valueEntry.getValue().getBsonType().name());
					columnList.add(tableColumnDto);
				}
			}
		}
		return columnList;
	}
	
	@Override
	public DsTypeEnum dsType() {
		return DsTypeEnum.mongo;
	}

	/**
	 * <p>创建连接</p>
	 * @author wookvn
	 * @datetime 2024-10-14 15:06:20
	 * @param ds: 数据源对象
	 * @return com.mongodb.client.MongoClient
	 **/
	private static MongoClient _createConnection(ReportDatasource ds) {
		// 创建连接字符串
		ConnectionString connectionString = new ConnectionString(ds.getUrl());
		int connectTimeout = Func.toInt(DsTypeEnum.mongo.jdbcConnectionPropTimeout(false).get("connectTimeout")),
				readTimeout = Func.toInt(DsTypeEnum.mongo.jdbcConnectionPropTimeout(false).get("socketTimeout"));

		// 创建MongoClientSettings.Builder
		MongoClientSettings.Builder settingsBuilder = MongoClientSettings.builder()
				.applyConnectionString(connectionString)
				// 连接超时时间，单位毫秒
				.applyToSocketSettings(builder -> builder.connectTimeout(connectTimeout, TimeUnit.MILLISECONDS))
				// 套接字读取超时时间，单位毫秒
				.applyToSocketSettings(builder -> builder.readTimeout(readTimeout, TimeUnit.MILLISECONDS));

		//兼容处理4.x版本的mongo
		MongoCredential credential = MongoCredential.createCredential(ds.getDsUser(), ds.getDsDb(), ds.getDsPassword().toCharArray());
		// 构建MongoClientSettings
		MongoClientSettings settings = settingsBuilder.credential(credential).build();
		return MongoClients.create(settings);
	}
}
