/**
 * 
 */
package org.springblade.etl.config;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson2.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.etl.properties.PreviewRedisProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisClusterConfiguration;
import org.springframework.data.redis.connection.RedisNode;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.jedis.JedisClientConfiguration;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.RedisSerializer;
import redis.clients.jedis.JedisPoolConfig;

import java.time.Duration;
import java.util.List;

/**
 * @author wookvn
 *
 */
@Slf4j
@Configuration
@EnableConfigurationProperties(PreviewRedisProperties.class)
public class JedisConfig {
	
	@Autowired
	private PreviewRedisProperties previewRedisProperties;

    @Bean("previewRedisTemplate")
    public RedisTemplate<String, Object> redisTemplate(JedisConnectionFactory jedisConnectionFactory) {
        JedisConnectionFactory connectionFactory = null;
        JedisClientConfiguration.JedisClientConfigurationBuilder builder = (JedisClientConfiguration.JedisClientConfigurationBuilder) JedisClientConfiguration.builder();
        //.usePooling().poolConfig(_jedisPoolConfig())
        JedisClientConfiguration jedisClientConfiguration = builder.connectTimeout(Duration.ofMillis(previewRedisProperties.getTimeout())).build();
        //集群环境
        if(CollectionUtil.isNotEmpty(previewRedisProperties.getNodes())) {
            RedisClusterConfiguration redisClusterConfiguration = new RedisClusterConfiguration();
            for(String hostPort : previewRedisProperties.getNodes()) {
                redisClusterConfiguration.addClusterNode(RedisNode.fromString(hostPort));
            }
            redisClusterConfiguration.setMaxRedirects(previewRedisProperties.getMaxAttempts());
            redisClusterConfiguration.setUsername(previewRedisProperties.getUsername());
            redisClusterConfiguration.setPassword(previewRedisProperties.getPassword());
            connectionFactory = new JedisConnectionFactory(redisClusterConfiguration, jedisClientConfiguration);
        } else {  //非集群环境
            RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration();
            redisStandaloneConfiguration.setUsername(previewRedisProperties.getUsername());
            redisStandaloneConfiguration.setHostName(previewRedisProperties.getHost());
            redisStandaloneConfiguration.setPassword(previewRedisProperties.getPassword());
            redisStandaloneConfiguration.setPort(previewRedisProperties.getPort());
            redisStandaloneConfiguration.setDatabase(previewRedisProperties.getDatabase());
            connectionFactory = new JedisConnectionFactory(redisStandaloneConfiguration, jedisClientConfiguration);
        }
        // 连接池初始化
        connectionFactory.afterPropertiesSet();
        RedisTemplate<String, Object> redisTemplate = new RedisTemplate<>();
        redisTemplate.setKeySerializer(RedisSerializer.string());
        redisTemplate.setValueSerializer(RedisSerializer.string());
        redisTemplate.setConnectionFactory(connectionFactory);
        redisTemplate.afterPropertiesSet();

        try {
            redisTemplate.hasKey("test_key");
            log.info("校验preview-redis实例正常运行");
        } catch (Exception e) {
            throw new ServiceException("校验preview-redis实例未正常运行，请检查|错误描述:" + e.getMessage());
        }

        return redisTemplate;
    }

    private JedisPoolConfig _jedisPoolConfig() {
        JedisPoolConfig jedisPoolConfig = new JedisPoolConfig();
        jedisPoolConfig.setMaxIdle(0);
        jedisPoolConfig.setMinIdle(5);
        jedisPoolConfig.setMaxTotal(5);
        jedisPoolConfig.setMaxWait(Duration.ofSeconds(2));
        jedisPoolConfig.setTestWhileIdle(true);
        jedisPoolConfig.setTimeBetweenEvictionRuns(Duration.ofMillis(30000));
        return jedisPoolConfig;
    }
}