/**
 * 
 */
package org.springblade.etl.connector.sink;

import com.alibaba.druid.util.JdbcUtils;
import lombok.extern.slf4j.Slf4j;
import org.springblade.bi.client.DataSourceClient;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.datasource.transform.TransformContext;
import org.springblade.etl.connector.sink.config.PlatomixJdbcSinkBaseConfig;
import org.springblade.etl.entity.bi.ReportDatasource;
import org.springblade.etl.enums.DataWriteMode;
import org.springblade.etl.service.bi.ReportDatasourceService;
import org.springblade.etl.util.EtlJsonUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;

/**
 * @author wookvn
 *
 */
@Slf4j
@Component
public class CommonJdbcSink  {

	@Autowired
	private ReportDatasourceService reportDatasourceService;
	
	@Autowired
	private TransformContext transformContext;

	@Autowired
	private DialectContext dialectContext;

	@Autowired
	private DataSourceClient dataSourceClient;
	
	/**
	 * jdbc - sink 前置操作
	 * @param configJson
	 */
	public void prepare(String configJson) {
		log.info("开始执行Sink的前置操作");
		PlatomixJdbcSinkBaseConfig sinkBaseConfig = EtlJsonUtil.parseJsonString(configJson, PlatomixJdbcSinkBaseConfig.class);
		if(sinkBaseConfig.getAutoCreateTable()) {
			log.info("自动建表开始|dsId={},tablename={}", sinkBaseConfig.getDatasourceId(), sinkBaseConfig.getTableName());
			ReportDatasource sinkDs = getDs(sinkBaseConfig.getDatasourceId());
			ReportDatasource sourceDs = null;
			//source没有JDBC数据源，比如：http-source/kafka-source等
			if(sinkBaseConfig.getSourceDatasourceId() == null) {
				sourceDs = new ReportDatasource();
				sourceDs.setDsType(DsTypeEnum.common.getValue());
			}else {
				sourceDs = getDs(sinkBaseConfig.getSourceDatasourceId());
			}
			//mongodb不用自动建表，它会自己操作的
			if (sinkDs.getDsTypeSyl() == DsTypeEnum.mongo){
				return;
			}
			long startTime = System.currentTimeMillis();
			transformContext.transformAndCreatTable(sourceDs, sinkDs, 
					sinkBaseConfig.getGeneralColumnList(), sinkBaseConfig.getSourceTableName(), sinkBaseConfig.getTableName(), sinkBaseConfig.getSinkTableComment(),true);
			log.info("自动建表结束|cost:{}ms", (System.currentTimeMillis() - startTime));
			//表创建完成，需要清理数据源缓存
			dataSourceClient.refreshDsCache(sinkDs.getId());
		}else if(sinkBaseConfig.getWriteMode() == DataWriteMode.TRUNCATE_INSERT) {
			log.info("清空数据开始|dsId={},tablename={}", sinkBaseConfig.getDatasourceId(), sinkBaseConfig.getTableName());
			Connection connection = null;
			Statement statement = null;
			try {
				ReportDatasource sourceModel = getDs(sinkBaseConfig.getDatasourceId());
				//mongo不支持清空数据
				if(sourceModel.getDsTypeSyl() == DsTypeEnum.mongo) {
					return;
				}
				Properties props = new Properties();
				props.put("user", sourceModel.getDsUser());
				props.put("password", sourceModel.getDsPassword());
				connection = DynamicDataSourceSupport.createConnection(sourceModel);
				statement = connection.createStatement();
				String truncateSql = String.format("truncate table %s", dialectContext.getDialect(sourceModel.getDsTypeSyl()).dbTableName(sourceModel, sinkBaseConfig.getTableName()));
				log.info("truncateSql:{}", truncateSql);
				statement.execute(truncateSql);
			} catch (Exception e) {
				log.error(e.getMessage());
				throw new ServiceException("执行Sink【前置操作:清空表】失败|"+e.getMessage(), e);
			} finally {
				JdbcUtils.close(connection);
				JdbcUtils.close(statement);
			}
		}
	}
	
	/**
	 * 获取数据源
	 * @param databaseId
	 * @return
	 */
	private ReportDatasource getDs(Long databaseId) {
		ReportDatasource sourceModel = reportDatasourceService.getById(databaseId);
		if(sourceModel == null) {
			throw new ServiceException("未查询到数据源|datasourceId="+databaseId);
		}
		return sourceModel;
	}
}
