/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.etl.connector.source.config;

import lombok.Data;
import lombok.EqualsAndHashCode;
import org.codehaus.jackson.annotate.JsonProperty;
import org.springblade.etl.connector.base.SeatunnelConfig;

/**
 * mongo的配置
 * org.apache.seatunnel.connectors.seatunnel.mongodb.config.MongodbConfig
 * @author wookvn
 * @datetime 2024-10-29 17:33:09
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class SeatunnelMongoSourceConfig extends SeatunnelConfig {

    /**
     *
     */
    private static final long serialVersionUID = 1L;

    private String uri;

    private String database;

    private String collection;

    private MongoSchemaField schema;

    @JsonProperty("match.query")
    private String matchQuery;

    @JsonProperty("match.projection")
    private String matchProjection;

    /**
     * 设置每个批处理从服务器获取的文档数量。设置合适的批处理大小可以提高查询性能，避免一次获取大量数据带来的内存压力。
     * 默认值：2048
     */
    @JsonProperty("fetch.size")
    private Integer fetchSize;

    /**
     * 该参数是MongoDB的查询选项，用于限制查询操作的最大执行时间。maxTimeMin的取值以分钟为单位。如果查询的执行时间超过了设置的时间限制，MongoDB将终止操作并返回错误。
     * 默认值：600
     */
    @JsonProperty("max.time-min")
    private Long maxTimeMin;
}
