/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.etl.engine.datax.dto;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * mongodb插件的column参数
 *
 * @author wookvn
 * @datetime 2024-10-29 21:30:07
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
public class DataxMongoColumn implements Serializable {

    private String name;

    private String type;

    private String spliter;

    /**
     * mongodb的数据类型到datax的映射
     */
    public transient final static Map<String, String> MONGO_TYPE_TO_DATAX_MAP = new HashMap<>();

    static{
        MONGO_TYPE_TO_DATAX_MAP.put("OBJECT_ID", "string");
        MONGO_TYPE_TO_DATAX_MAP.put("STRING", "string");
        MONGO_TYPE_TO_DATAX_MAP.put("INT32", "int");
        MONGO_TYPE_TO_DATAX_MAP.put("INT64", "bigint");
        MONGO_TYPE_TO_DATAX_MAP.put("ARRAY", "string");
        MONGO_TYPE_TO_DATAX_MAP.put("DOUBLE", "double");
        MONGO_TYPE_TO_DATAX_MAP.put("DECIMAL128", "double");
        MONGO_TYPE_TO_DATAX_MAP.put("DATE", "date");
        MONGO_TYPE_TO_DATAX_MAP.put("NULL", "string");
        MONGO_TYPE_TO_DATAX_MAP.put("DATE_TIME", "date");
        MONGO_TYPE_TO_DATAX_MAP.put("TIMESTAMP", "date");
        MONGO_TYPE_TO_DATAX_MAP.put("BOOLEAN", "boolean");
    }
}
